'use strict';

var sinon = require('sinon');
var has = sinon.match.has;
var any = sinon.match.any;
var defined = sinon.match.defined;
var chai = require('chai');
var sinonChai = require('sinon-chai');
chai.use(sinonChai);
var expect = chai.expect;
var mandrill = require('mandrill-api/mandrill');
var Email = require('../../lib/email/email');

describe('email', function() {
    describe('send', function() {
        var messagesMock;

        beforeEach(function() {
            messagesMock = {sendTemplate: sinon.spy()};
            sinon.stub(mandrill, 'Mandrill').returns({messages: messagesMock});
        });
        afterEach(function() {
            mandrill.Mandrill.restore();
        });

        it('should send an e-mail with mandrill using template', function() {
            var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});
            var sendTemplate = 'the template to be sent';

            emailHandler.send(sendTemplate);

            expect(messagesMock.sendTemplate).to.have.been.calledWith(sendTemplate);
        });
    });

    describe('callbacks', function() {
        var callback = sinon.spy();

        beforeEach(function() {
            callback.reset();
        });

        describe('when mandrill is on success', function() {
            function mandrillResultWithStatus(status) {
                return [{status: status}];
            }
            function whenMandrillIsOnSuccess(result) {
                var mandrillStubbed = new mandrill.Mandrill('mandrillApiKey');

                beforeEach(function() {
                    sinon.stub(mandrillStubbed.messages, 'sendTemplate', function(email, onSuccess) {
                        return onSuccess(result);
                    });
                    sinon.stub(mandrill, 'Mandrill').returns(mandrillStubbed);
                });
                afterEach(function() {
                    mandrillStubbed.messages.sendTemplate.restore();
                    mandrill.Mandrill.restore();
                });

            }
            function itShouldReturnSuccess(result) {
                it('should return success', function() {
                    var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});

                    emailHandler.send('template', callback);

                    expect(callback).to.have.been.calledWith(null, result);
                });
            }
            function itShouldReturnError(mandrillStatus) {
                it('should return error', function() {
                    var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});

                    emailHandler.send('template', callback);

                    expect(callback).to.have.been.calledWithMatch(has('code', -1));
                    expect(callback).to.have.been.calledWithMatch(
                        has('message', 'Mandrill reports a bad status for e-mail sent: ' + mandrillStatus)
                    );
                    expect(callback).to.have.been.calledWithMatch(has('name', 'MandrillResultError'));
                    expect(callback).to.have.been.calledWithMatch(has('status', 'error'));
                    expect(callback).to.not.have.been.calledWithMatch(any, defined);
                });
            }
            describe('with status sent', function() {
                var sent = mandrillResultWithStatus('sent');
                whenMandrillIsOnSuccess(sent);
                itShouldReturnSuccess(sent);
            });

            describe('with status queued', function() {
                var queued = mandrillResultWithStatus('queued');
                whenMandrillIsOnSuccess(queued);
                itShouldReturnSuccess(queued);
            });

            describe('with another status', function() {
                var weird = mandrillResultWithStatus('weird_status');
                whenMandrillIsOnSuccess(weird);
                itShouldReturnError('weird_status');
            });

            describe('with no result', function() {
                whenMandrillIsOnSuccess(null);
                itShouldReturnError('null');
            });
        });

        describe('when mandrill is on error', function() {
            var mandrillStubbed = new mandrill.Mandrill('mandrillApiKey');
            var mandrillError = 'error';

            beforeEach(function() {
                sinon.stub(mandrillStubbed.messages, 'sendTemplate', function(email, onSuccess, onError) {
                    return onError(mandrillError);
                });
                sinon.stub(mandrill, 'Mandrill').returns(mandrillStubbed);
            });
            afterEach(function() {
                mandrillStubbed.messages.sendTemplate.restore();
                mandrill.Mandrill.restore();
            });

            it('should return error', function() {
                var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});

                emailHandler.send('template', callback);

                expect(callback).to.have.been.calledWith(mandrillError);
            });
        });
    });

    describe('constructors', function() {
        var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});

        it('should handle prepare email', function() {
            var email = new emailHandler.SendTemplate('default', [{
                name: 'TITLE',
                content: 'some hard-coded title'
            }], {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                }
            });
            expect(email.template_name).to.be.equal('default');
        });
        it('should handle accept empty values', function() {
            var email = new emailHandler.SendTemplate('default', undefined, {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                }
            });
            expect(email.template_name).to.be.equal('default');
        });

        it('should have EmailError', function() {
            var emailError = new emailHandler.EmailError();
            expect(emailError).to.have.property('name', 'EmailError');
        });
    });
});
